"use strict";

require("core-js/modules/es.object.define-property");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useTracking;

var _deepmerge = _interopRequireDefault(require("deepmerge"));

var _react = require("react");

var _withTrackingComponentDecorator = require("./withTrackingComponentDecorator");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function useTracking() {
  var trackingContext = (0, _react.useContext)(_withTrackingComponentDecorator.ReactTrackingContext);

  if (!(trackingContext && trackingContext.tracking)) {
    throw new Error('Attempting to call `useTracking` ' + 'without a ReactTrackingContext present. Did you forget to wrap the top of ' + 'your component tree with `track`?');
  }

  return (0, _react.useMemo)(function () {
    return {
      getTrackingData: function getTrackingData() {
        return trackingContext.tracking.data;
      },
      trackEvent: function trackEvent(data) {
        return trackingContext.tracking.dispatch((0, _deepmerge["default"])(trackingContext.tracking.data, data));
      }
    };
  }, [trackingContext.tracking.data]);
}